@@ -246,16 +246,6 @@ sub DistinctQuery {
     my $statementref = shift;
     my $sb = shift;
 
-    # when we have group by clause then the result set is distinct as
-    # it must contain only columns we group by or results of aggregate
-    # functions which give one result per group, so we can skip DISTINCTing
-    if ( my $group = $sb->_GroupClause ) {
-        $$statementref = "SELECT main.* FROM $$statementref";
-        $$statementref .= $group;
-        $$statementref .= $sb->_OrderClause;
-        return;
-    }
-
     my $table = $sb->Table;
 
     if ($sb->_OrderClause =~ /(?<!main)\./) {
@@ -272,7 +262,7 @@ sub DistinctQuery {
         ];
         my $group = $sb->_GroupClause;
         my $order = $sb->_OrderClause;
-        $$statementref = "SELECT main.* FROM ( SELECT main.id FROM $$statementref $group $order ) distinctquery, $table main WHERE (main.id = distinctquery.id)";
+        $$statementref = "SELECT main.* FROM ( SELECT main.id, row_number() over( $order ) sortorder FROM $$statementref $group ) distinctquery, $table main WHERE (main.id = distinctquery.id) ORDER BY distinctquery.sortorder";
     } else {
         # Wrapp select query in a subselect as Oracle doesn't allow
         # DISTINCT against CLOB/BLOB column types.
@@ -307,6 +297,26 @@ sub DatabaseVersion {
     return ''. ORA_OCI;
 }
 
+sub Fields {
+    my $self  = shift;
+    my $table = shift;
+
+    my $cache = \%DBIx::SearchBuilder::Handle::FIELDS_IN_TABLE;
+    unless ( $cache->{ lc $table } ) {
+        # uc(table) required as oracle stores UC names in information tables
+        # and lookup clauses are case sensetive
+        my $sth = $self->dbh->column_info( undef, undef, uc($table), '%' )
+            or return ();
+        my $info = $sth->fetchall_arrayref({});
+        # TODO: not sure why results are lower case, probably NAME_ls affects it
+        # we should check it out at some point
+        foreach my $e ( sort {$a->{'ordinal_position'} <=> $b->{'ordinal_position'}} @$info ) {
+            push @{ $cache->{ lc $e->{'table_name'} } ||= [] }, lc $e->{'column_name'};
+        }
+    }
+    return @{ $cache->{ lc $table } || [] };
+}
+
 1;
 
 __END__
